@@ -231,7 +231,8 @@ convert_EImage_to_XImage (Lisp_Object device, int width, int height,
   if (vis->class == PseudoColor)
     {
       unsigned long pixarray[256];
-      int pixcount, n;
+      int pixcount;
+      unsigned int n;
       /* use our quantize table to allocate the colors */
       pixcount = 32;
       *pixtbl = xnew_array (unsigned long, pixcount);
@@ -432,7 +433,7 @@ x_finalize_image_instance (Lisp_Image_Instance *p)
 	}
       else
 	{
-	  int i;
+	  unsigned int i;
 	  if (IMAGE_INSTANCE_PIXMAP_TIMEOUT (p))
 	    disable_glyph_animated_timeout (IMAGE_INSTANCE_PIXMAP_TIMEOUT (p));
 
@@ -2778,7 +2779,7 @@ x_tab_control_redisplay (Lisp_Object image_instance)
 		     but I couldn't find it. */
 		  Lisp_Object old_selected =gui_item_list_find_selected
 		    (XCDR (IMAGE_INSTANCE_WIDGET_ITEMS (ii)));
-		  Arg al [1];
+		  Arg al [2];
 		  char* name;
 		  unsigned int num_children, i;
 		  Widget* children;
@@ -2794,7 +2795,9 @@ x_tab_control_redisplay (Lisp_Object image_instance)
 		      if (!strcmp (XtName (children [i]), name))
 			{
 			  XtSetArg (al [0], XtNtopWidget, children [i]);
-			  XtSetValues (IMAGE_INSTANCE_X_WIDGET_ID (ii), al, 1);
+			  XtSetArg (al [1], XtNhighlightWidget,
+				    children [i]);
+			  XtSetValues (IMAGE_INSTANCE_X_WIDGET_ID (ii), al, 2);
 			  break;
 			}
 		    }
