@@ -2,9 +2,9 @@
 emacs_is_beta=
 emacs_major_version=21
 emacs_minor_version=4
-emacs_beta_version=8
-xemacs_codename="Honest Recruiter"
-emacs_kitname=""
+emacs_beta_version=9
+xemacs_codename="Informed Management"
+emacs_kit_version=
 infodock_major_version=4
 infodock_minor_version=0
 infodock_build_version=8
